@@ -15,26 +15,21 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
- *)
+*)
 
 (** Typesafe constructors and printers for SVG documents.
 
     @see <http://www.w3.org/TR/SVG> W3C Recommendation *)
 
 (** Concrete implementation of SVG typesafe constructors *)
-module M : Svg_sigs.T with type Xml.uri = Xml.uri
-                      and type Xml.event_handler = Xml.event_handler
-                      and type Xml.attrib = Xml.attrib
-                      and type Xml.elt = Xml.elt
-                      and type 'a Xml.wrap = 'a
-                      and type 'a wrap = 'a
+module M : Svg_sigs.Make(Xml).T
 
 (** Simple printer for SVG documents *)
 module P : Xml_sigs.Typed_simple_printer with type 'a elt := 'a M.elt
-				        and type doc := M.doc
+                                          and type doc := M.doc
 
 (** Parametrized stream printer for SVG documents *)
 module Make_printer(O : Xml_sigs.Output) :
   Xml_sigs.Typed_printer with type out := O.out
-                         and type 'a elt := 'a M.elt
-                         and type doc := M.doc
+                          and type 'a elt := 'a M.elt
+                          and type doc := M.doc
