@@ -15,7 +15,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
- *)
+*)
 
 module type T = sig
 
@@ -31,23 +31,19 @@ module type T = sig
 
   (** {1 Abstraction over XML's types} *)
 
-  type 'a attrib
+  type +'a attrib
 
   type 'a wrap
+  type 'a list_wrap
 
   type +'a elt
 
-  type +'a elts
-
   type ('a, 'b) nullary = ?a: (('a attrib) list) -> unit -> 'b elt
 
   type ('a, 'b, 'c) unary = ?a: (('a attrib) list) -> 'b elt wrap -> 'c elt
 
   type ('a, 'b, 'c) star =
-      ?a: (('a attrib) list) -> ('b elt) list wrap -> 'c elt
-
-  type ('a, 'b, 'c) plus =
-      ?a: (('a attrib) list) -> 'b elt wrap -> ('b elt) list wrap -> 'c elt
+    ?a: (('a attrib) list) -> ('b elt) list_wrap -> 'c elt
 
   (* to be processed by a script *)
   type altglyphdef_content =
@@ -154,6 +150,12 @@ module type T = sig
 
   val a_textlength : Unit.length wrap -> [> | `TextLength ] attrib
 
+  val a_text_anchor : [< `Start | `Middle | `End | `Inherit ] wrap -> [> | `Text_Anchor ] attrib
+
+  val a_text_decoration : [< `None | `Underline | `Overline | `Line_through | `Blink | `Inherit ] wrap -> [> | `Text_Decoration ] attrib
+
+  val a_text_rendering : [< `Auto | `OptimizeSpeed | `OptimizeLegibility | `GeometricPrecision | `Inherit ] wrap -> [> | `Text_Rendering ] attrib
+
   val a_rotate : numbers wrap -> [> | `Rotate ] attrib
 
   val a_startoffset : Unit.length wrap -> [> | `StartOffset ] attrib
@@ -183,11 +185,11 @@ module type T = sig
 
   val a_renderingindent :
     [<
-    | `Auto
-    | `Perceptual
-    | `Relative_colorimetric
-    | `Saturation
-    | `Absolute_colorimetric ] wrap -> [> | `Rendering_Indent ] attrib
+      | `Auto
+      | `Perceptual
+      | `Relative_colorimetric
+      | `Saturation
+      | `Absolute_colorimetric ] wrap -> [> | `Rendering_Indent ] attrib
 
   val a_gradientunits :
     [< `UserSpaceOnUse | `ObjectBoundingBox ] wrap ->
@@ -237,23 +239,23 @@ module type T = sig
 
   val a_in :
     [<
-    | `SourceGraphic
-    | `SourceAlpha
-    | `BackgroundImage
-    | `BackgroundAlpha
-    | `FillPaint
-    | `StrokePaint
-    | `Ref of string ] wrap -> [> | `In ] attrib
+      | `SourceGraphic
+      | `SourceAlpha
+      | `BackgroundImage
+      | `BackgroundAlpha
+      | `FillPaint
+      | `StrokePaint
+      | `Ref of string ] wrap -> [> | `In ] attrib
 
   val a_in2 :
     [<
-    | `SourceGraphic
-    | `SourceAlpha
-    | `BackgroundImage
-    | `BackgroundAlpha
-    | `FillPaint
-    | `StrokePaint
-    | `Ref of string ] wrap -> [> | `In2 ] attrib
+      | `SourceGraphic
+      | `SourceAlpha
+      | `BackgroundImage
+      | `BackgroundAlpha
+      | `FillPaint
+      | `StrokePaint
+      | `Ref of string ] wrap -> [> | `In2 ] attrib
 
   val a_aizmuth : float wrap -> [> | `Azimuth ] attrib
 
@@ -287,8 +289,6 @@ module type T = sig
 
   val a_tablevalues : numbers wrap -> [> | `TableValues ] attrib
 
-  val a_slope : number wrap -> [> | `Slope ] attrib
-
   val a_intercept : number wrap -> [> | `Intercept ] attrib
 
   val a_amplitude : number wrap -> [> | `Amplitude ] attrib
@@ -363,7 +363,9 @@ module type T = sig
 
   val a_xlinkshow : [< | `New | `Replace ] wrap -> [> | `Xlink_show ] attrib
 
-  val a_xlinkactuate : [< | `OnRequest ] wrap -> [> | `Xlink_actuate ] attrib
+  val a_xlinkactuate :
+    [< | `OnRequest | `OnLoad | `Other | `None ] wrap
+    -> [> | `Xlink_actuate ] attrib
 
   val a_target : string wrap -> [> | `Xlink_target ] attrib
 
@@ -459,17 +461,17 @@ module type T = sig
 
   val a_k : string wrap -> [> | `K ] attrib
 
-  val a_fontfamily : string wrap -> [> | `FontFamily ] attrib
+  val a_fontfamily : string wrap -> [> | `Font_Family ] attrib
 
-  val a_fontstyle : string wrap -> [> | `FontStyle ] attrib
+  val a_fontstyle : string wrap -> [> | `Font_Style ] attrib
 
-  val a_fontvariant : string wrap -> [> | `FontVariant ] attrib
+  val a_fontvariant : string wrap -> [> | `Font_Variant ] attrib
 
-  val a_fontweight : string wrap -> [> | `FontWeight ] attrib
+  val a_fontweight : string wrap -> [> | `Font_Weight ] attrib
 
-  val a_fontstretch : string wrap -> [> | `FontStretch ] attrib
+  val a_fontstretch : string wrap -> [> | `Font_Stretch ] attrib
 
-  val a_fontsize : string wrap -> [> | `FontSize ] attrib
+  val a_fontsize : string wrap -> [> | `Font_Size ] attrib
 
   val a_unicoderange : string wrap -> [> | `UnicodeRange ] attrib
 
@@ -527,50 +529,50 @@ module type T = sig
 
   val a_name : string wrap -> [> | `Name ] attrib
 
+  val a_alignment_baseline :
+    [< | `Auto | `Baseline | `Before_edge | `Text_before_edge | `Middle
+       | `Central | `After_edge | `Text_after_edge | `Ideographic
+       | `Alphabetic | `Hanging | `Mathematical | `Inherit ] wrap ->
+    [> | `Alignment_Baseline ] attrib
+
+  val a_dominant_baseline :
+    [< | `Auto | `Use_script | `No_change | `Reset_size | `Ideographic
+       | `Alphabetic | `Hanging | `Mathematical | `Central | `Middle
+       | `Text_after_edge | `Text_before_edge | `Inherit ] wrap ->
+    [> | `Dominant_Baseline ] attrib
+
+  (** Javascript events *)
+
   val a_onabort : Xml.event_handler  -> [> | `OnAbort ] attrib
-
   val a_onactivate : Xml.event_handler  -> [> | `OnActivate ] attrib
-
   val a_onbegin : Xml.event_handler  -> [> | `OnBegin ] attrib
-
-  val a_onclick : Xml.event_handler  -> [> | `OnClick ] attrib
-
   val a_onend : Xml.event_handler  -> [> | `OnEnd ] attrib
-
   val a_onerror : Xml.event_handler  -> [> | `OnError ] attrib
-
   val a_onfocusin : Xml.event_handler  -> [> | `OnFocusIn ] attrib
-
   val a_onfocusout : Xml.event_handler  -> [> | `OnFocusOut ] attrib
-
   val a_onload : Xml.event_handler  -> [> | `OnLoad ] attrib
-
-  val a_onmousedown : Xml.event_handler  -> [> | `OnMouseDown ] attrib
-
-  val a_onmouseup : Xml.event_handler  -> [> | `OnMouseUp ] attrib
-
-  val a_onmouseover : Xml.event_handler  -> [> | `OnMouseOver ] attrib
-
-  val a_onmouseout : Xml.event_handler  -> [> | `OnMouseOut ] attrib
-
-  val a_onmousemove : Xml.event_handler  -> [> | `OnMouseMove ] attrib
-
   val a_onrepeat : Xml.event_handler  -> [> | `OnRepeat ] attrib
-
   val a_onresize : Xml.event_handler  -> [> | `OnResize ] attrib
-
   val a_onscroll : Xml.event_handler  -> [> | `OnScroll ] attrib
-
   val a_onunload : Xml.event_handler  -> [> | `OnUnload ] attrib
-
   val a_onzoom : Xml.event_handler  -> [> | `OnZoom ] attrib
 
+  (** Javascript mouse events *)
+
+  val a_onclick : Xml.mouse_event_handler  -> [> | `OnClick ] attrib
+  val a_onmousedown : Xml.mouse_event_handler  -> [> | `OnMouseDown ] attrib
+  val a_onmouseup : Xml.mouse_event_handler  -> [> | `OnMouseUp ] attrib
+  val a_onmouseover : Xml.mouse_event_handler  -> [> | `OnMouseOver ] attrib
+  val a_onmouseout : Xml.mouse_event_handler  -> [> | `OnMouseOut ] attrib
+  val a_onmousemove : Xml.mouse_event_handler  -> [> | `OnMouseMove ] attrib
+
+
   val metadata :
-    ?a: ((metadata_attr attrib) list) -> Xml.elt list wrap -> [> | metadata] elt
+    ?a: ((metadata_attr attrib) list) -> Xml.elt list_wrap -> [> | metadata] elt
 
   val foreignobject :
     ?a: ((foreignobject_attr attrib) list) ->
-    Xml.elt list wrap -> [> | foreignobject] elt
+    Xml.elt list_wrap -> [> | foreignobject] elt
 
   val a_stopcolor : color wrap -> [> | `Stop_Color ] attrib
 
@@ -648,11 +650,12 @@ module type T = sig
 
   val altglyphdef :
     ([< | altglyphdef_attr], [< | altglyphdef_content], [> | altglyphdef])
-	unary
+      unary
 
+  (* theoretically a plus, simplified into star *)
   val altglyphitem :
     ([< | altglyphitem_attr], [< | altglyphitem_content], [> | altglyphitem
-							  ]) plus
+                                                          ]) star
 
   val glyphref : ([< | glyphref_attr], [> | glyphref]) nullary
 
@@ -660,7 +663,7 @@ module type T = sig
 
   val colorprofile :
     ([< | colorprofile_attr], [< | colorprofile_content], [> | colorprofile
-							  ]) star
+                                                          ]) star
 
   val lineargradient :
     ([< | lineargradient_attr], [< | lineargradient_content],
@@ -687,11 +690,11 @@ module type T = sig
 
   val fepointlight :
     ([< | fepointlight_attr], [< | fepointlight_content], [> | fepointlight
-							  ]) star
+                                                          ]) star
 
   val fespotlight :
     ([< | fespotlight_attr], [< | fespotlight_content], [> | fespotlight])
-	star
+      star
 
   val feblend :
     ([< | feblend_attr], [< | feblend_content], [> | feblend]) star
@@ -718,7 +721,7 @@ module type T = sig
 
   val fecomposite :
     ([< | fecomposite_attr], [< | fecomposite_content], [> | fecomposite])
-	star
+      star
 
   val feconvolvematrix :
     ([< | feconvolvematrix_attr], [< | feconvolvematrix_content],
@@ -747,7 +750,7 @@ module type T = sig
 
   val femorphology :
     ([< | femorphology_attr], [< | femorphology_content], [> | femorphology
-							  ]) star
+                                                          ]) star
 
   val feoffset :
     ([< | feoffset_attr], [< | feoffset_content], [> | feoffset]) star
@@ -760,7 +763,7 @@ module type T = sig
 
   val feturbulence :
     ([< | feturbulence_attr], [< | feturbulence_content], [> | feturbulence
-							  ]) star
+                                                          ]) star
 
   val cursor : ([< | cursor_attr], [< | cursor_content], [> | cursor]) star
 
@@ -784,7 +787,7 @@ module type T = sig
 
   val animatecolor :
     ([< | animatecolor_attr], [< | animatecolor_content], [> | animatecolor
-							  ]) star
+                                                          ]) star
 
   val animatetransform :
     ([< | animatetransform_attr], [< | animatetransform_content],
@@ -796,7 +799,7 @@ module type T = sig
 
   val missingglyph :
     ([< | missingglyph_attr], [< | missingglyph_content], [> | missingglyph
-							  ]) star
+                                                          ]) star
 
   val hkern : ([< | hkern_attr], [> | hkern]) nullary
 
@@ -806,11 +809,11 @@ module type T = sig
 
   val fontfacesrc :
     ([< | fontfacesrc_attr], [< | fontfacesrc_content], [> | fontfacesrc])
-	star
+      star
 
   val fontfaceuri :
     ([< | fontfaceuri_attr], [< | fontfaceuri_content], [> | fontfaceuri])
-	star
+      star
 
   val fontfaceformat :
     ([< | fontfaceformat_attr], [> | fontfaceformat]) nullary
@@ -842,7 +845,7 @@ module type T = sig
         If it is a standard SVG node which is missing,
         please report to the Ocsigen team.
     *)
-    val node : string -> ?a:'a attrib list -> 'b elt list wrap -> 'c elt
+    val node : string -> ?a:'a attrib list -> 'b elt list_wrap -> 'c elt
 
     (** Insert an XML node without children
         that is not implemented in this module.
@@ -886,3 +889,34 @@ module type T = sig
 
 
 end
+
+(** {2 Signature functors} *)
+(** See {% <<a_manual chapter="functors"|the manual of the functorial interface>> %}. *)
+
+(** Signature functor for {!Svg_f.MakeWrapped}. *)
+module MakeWrapped
+    (W : Xml_wrap.T)
+    (Xml : Xml_sigs.Wrapped) :
+sig
+
+  (** See {!modtype:Svg_sigs.T}. *)
+  module type T = T
+    with type Xml.uri = Xml.uri
+     and type Xml.event_handler = Xml.event_handler
+     and type Xml.mouse_event_handler = Xml.mouse_event_handler
+     and type Xml.keyboard_event_handler = Xml.keyboard_event_handler
+     and type Xml.attrib = Xml.attrib
+     and type Xml.elt = Xml.elt
+     and type 'a Xml.wrap = 'a W.t
+     and type 'a wrap = 'a W.t
+     and type 'a Xml.list_wrap = 'a W.tlist
+     and type 'a list_wrap = 'a W.tlist
+end
+
+(** Signature functor for {!Svg_f.Make}. *)
+module Make(Xml : Xml_sigs.T) :
+sig
+
+  (** See {!modtype:Svg_sigs.MakeWrapped} and {!modtype:Svg_sigs.T}. *)
+  module type T = MakeWrapped(Xml_wrap.NoWrap)(Xml).T
+end
