@@ -17,7 +17,7 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
- *)
+*)
 
 (** SVG types with variants. (See also {!Svg.M}) *)
 
@@ -29,8 +29,6 @@
     and the interface is very low level and do not take deeply into account
     the needs of SVG elements. *)
 
-(** See {!Xhtml.M} for a description of the transcription's principles. *)
-
 (* Some attribtes and elements are tagged with XXX: they
    may be improved and do not match completely the SVG spec *)
 
@@ -164,7 +162,7 @@ type animation_addition_attr = [ | `Additive | `Accumulate ]
 
 type presentation_attr =
   [
-    | `Alignement_Baseline
+    | `Alignment_Baseline
     | `Baseline_Shift
     | `Clip
     | `Clip_Path
@@ -177,7 +175,7 @@ type presentation_attr =
     | `Cursor
     | `Direction
     | `Display
-    | `Dominant_baseline
+    | `Dominant_Baseline
     | `Enable_background
     | `Fill
     | `Fill_opacity
@@ -748,6 +746,7 @@ type text_attr =
     | core_attr
     | graphical_event_attr
     | presentation_attr
+    | `Class
     | `Transform
     | `LengthAdjust
     | `X_list
@@ -793,24 +792,6 @@ type tspan_attr =
     | `LengthAdjust
   ]
 
-type tspan_attribute =
-  [
-    | conditional_processing_attr
-    | core_attr
-    | graphical_event_attr
-    | presentation_attr
-    | `Class
-    | `Style
-    | `ExternalResourcesRequired
-    | `X_list
-    | `Y_list
-    | `Dx_list
-    | `Dy_list
-    | `Rotate
-    | `TextLength
-    | `LengthAdjust
-  ]
-
 type tref = [ | `Tref ]
 
 (* star *)
@@ -895,7 +876,7 @@ type altglyphdef_attr = [ | core_attr ]
 
 type altglyphitem = [ | `AltGlyphItem ]
 
-(* plus *)
+
 type altglyphitem_content = [ | `glyphRef ]
 
 type altglyphitem_attr = [ | core_attr ]
@@ -1808,7 +1789,7 @@ type missingglyph_content =
     | `Cursor
     | `Filter
     | `Font
-    | `FontFace
+    | `Font_Face
     | `ForeignObject
     | `Image
     | `Marker
@@ -1844,7 +1825,7 @@ type vkern = [ | `Vkern ]
 (* nullary *)
 type vkern_attr = [ | core_attr | `U1 | `G1 | `U2 | `G2 | `K ]
 
-type fontface = [ | `FontFace ]
+type fontface = [ | `Font_Face ]
 
 (* nullary *)
 type fontface_content = [ | descriptive_element | `Font_Face_Src ]
@@ -1852,12 +1833,12 @@ type fontface_content = [ | descriptive_element | `Font_Face_Src ]
 type fontface_attr =
   [
     | core_attr
-    | `FontFamily
-    | `FontStyle
-    | `FontVariant
-    | `FontWeight
-    | `FontStretch
-    | `FontSize
+    | `Font_Family
+    | `Font_Style
+    | `Font_Variant
+    | `Font_Weight
+    | `Font_Stretch
+    | `Font_Size
     | `UnicodeRange
     | `UnitsPerEm
     | `Panose1
@@ -1901,7 +1882,7 @@ type fontfaceuri_content = [ | `Font_Face_Format ]
 
 type fontfaceuri_attr = [ | core_attr | xlink_attr | `Xlink_href ]
 
-type fontfaceformat = [ | `Font_Face_Uri ]
+type fontfaceformat = [ | `Font_Face_Format ]
 
 (* nullary *)
 type fontfaceformat_attr = [ | core_attr | `String ]
