@@ -17,13 +17,14 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA 02111-1307, USA.
- *)
+*)
 
 (** Attributes *)
 
 module M = struct
 
   type 'a wrap = 'a
+  type 'a list_wrap = 'a list
 
   type uri = string
   let uri_of_string s = s
@@ -39,6 +40,8 @@ module M = struct
     | AStrL of separator * string list
   type attrib = aname * acontent
   type event_handler = string
+  type mouse_event_handler = string
+  type keyboard_event_handler = string
 
   let acontent (_, a) = a
   let aname (name, _) = name
@@ -49,13 +52,13 @@ module M = struct
   let space_sep_attrib name values = name, AStrL (Space, values)
   let comma_sep_attrib name values = name, AStrL (Comma, values)
   let event_handler_attrib name value = name, AStr value
+  let mouse_event_handler_attrib name value = name, AStr value
+  let keyboard_event_handler_attrib name value = name, AStr value
   let uri_attrib name value = name, AStr value
   let uris_attrib name values = name, AStrL (Space, values)
 
-  (* Deprecated alias. *)
-  let event_attrib = event_handler_attrib
 
-(** Element *)
+  (** Element *)
 
   type ename = string
   type econtent =
@@ -81,43 +84,43 @@ module M = struct
   let entity e = { elt = Entity e }
 
   let cdata s = (* GK *)
-  (* For security reasons, we do not allow "]]>" inside CDATA
-     (as this string is to be considered as the end of the cdata)
-  *)
-  let s' = "\n<![CDATA[\n"^
-    (Str.global_replace (Str.regexp (Str.quote "]]>")) "" s)
-    ^"\n]]>\n" in
-  encodedpcdata s'
+    (* For security reasons, we do not allow "]]>" inside CDATA
+       (as this string is to be considered as the end of the cdata)
+    *)
+    let s' = "\n<![CDATA[\n"^
+             (Str.global_replace (Str.regexp (Str.quote "]]>")) "" s)
+             ^"\n]]>\n" in
+    encodedpcdata s'
 
   let cdata_script s = (* GK *)
-  (* For security reasons, we do not allow "]]>" inside CDATA
-     (as this string is to be considered as the end of the cdata)
-  *)
+    (* For security reasons, we do not allow "]]>" inside CDATA
+       (as this string is to be considered as the end of the cdata)
+    *)
     let s' = "\n//<![CDATA[\n"^
-      (Str.global_replace (Str.regexp (Str.quote "]]>")) "" s)
-      ^"\n//]]>\n" in
+             (Str.global_replace (Str.regexp (Str.quote "]]>")) "" s)
+             ^"\n//]]>\n" in
     encodedpcdata s'
 
   let cdata_style s = (* GK *)
-  (* For security reasons, we do not allow "]]>" inside CDATA
-     (as this string is to be considered as the end of the cdata)
-  *)
+    (* For security reasons, we do not allow "]]>" inside CDATA
+       (as this string is to be considered as the end of the cdata)
+    *)
     let s' = "\n/* <![CDATA[ */\n"^
-      (Str.global_replace (Str.regexp (Str.quote "]]>")) "" s)
-      ^"\n/* ]]> */\n" in
+             (Str.global_replace (Str.regexp (Str.quote "]]>")) "" s)
+             ^"\n/* ]]> */\n" in
     encodedpcdata s'
 
   let leaf ?a name =
     { elt =
-	(match a with
-	  | Some a -> Leaf (name, a)
-	  | None -> Leaf (name, [])) }
+        (match a with
+         | Some a -> Leaf (name, a)
+         | None -> Leaf (name, [])) }
 
   let node ?a name children =
     { elt =
-	(match a with
-	  | Some a -> Node (name, a, children)
-	  | None -> Node (name, [], children)) }
+        (match a with
+         | Some a -> Node (name, a, children)
+         | None -> Node (name, [], children)) }
 
 
 end
